/*
 * iflib.h - Functions for processing if expressions
 *
 * Written 2001 by Werner Almesberger
 * Copyright 2001 EPFL-ICA, Network Robots
 */


#ifndef IFLIB_H
#define IFLIB_H

#include <stdio.h>

#include "location.h"
#include "data.h"
#include "tree.h"


DATA iflib_combine(const QDISC *qdisc);

/* Generate if expression from all filters on that qdisc */

void iflib_normalize(DATA *d);

/* Normalize && and || */

void iflib_reduce(DATA *d);

/* General optimizations */

void iflib_offset(DATA *d,int combine);

/*
 * Add and reduce explicit offset operators. If "combine" is non-zero,
 * iflib_offset also tries to combine offsets across sub-expressions.
 */

void iflib_actions(DATA *d);

/*
 * Convert actions to canonical format by moving policing and such pastall
 * matches.
 */

void iflib_arith(DATA *d);

/*
 * Perform various simple arithmetic optimizations.
 */

void iflib_not(DATA *d);

/*
 * Replace negations in logical expressions by re-arranging the expression
 * around || and &&.
 */

int expr_equal(DATA a,DATA b);

/*
 * Actually, this is a test for whether the expressions are _identical_, not
 * only equal. Returns non-zero if they are.
 */

int action_tree(DATA d);

/*
 * Returns non-zero if the sub-tree rooted at d consists exclusively of
 * policing operators, class selections, and logical operators combining
 * such things. I.e. any such sub-tree goes into the "actions" group at
 * the external interface, while all or part of a sub-tree for which
 * action_tree returns zero has to go into the "rules" group.
 */

typedef enum {
    sc_no,		/* not self_contained */
    sc_no_false,	/* not self-contained, expression is false */
    sc_no_true,		/* not self-contained, expression is true */
    sc_yes		/* is self-contained */
} SELF_CONTAINED;

SELF_CONTAINED self_contained(DATA d);

/*
 * A sub-tree is self_contained if it selects a class, or if any failed
 * conformance decisions lead to a self_contained subtree.
 *
 * Note: self_contained may incorrectly return 0 if the subtree is not an
 * action_tree.
 */

void iflib_cheap(LOCATION loc,DATA d);

/*
 * Refuse all operations that appear to be "expensive"
 */

void iflib_bit(FILE *file,DATA d);
void iflib_newbit(FILE *file,DATA d);


/* New action database handling from if_ext_act.c: */

void register_actions(DATA d);
void write_actions(FILE *file);
int action_number(DATA d);
void free_actions(void);


#define dump_failed(d) __dump_failed(d,__FILE__,__LINE__)
void __dump_failed(DATA d,const char *file,int line);

#endif /* IFLIB_H */
