/*
 * iflib_cheap.c - Normalize and reduce (optimize) an if expression
 *
 * Written 2001 by Werner Almesberger
 * Copyright 2001 Network Robots
 */


#include "config.h"
#include "error.h"
#include "data.h"
#include "op.h"
#include "iflib.h"


static void refuse_expensive(LOCATION loc,DATA d,int in_actions)
{
    if (!in_actions) {
	if (self_contained(d) == sc_yes) in_actions = 1;
	else if (action_tree(d))
		lerror(loc,"policing expressions not always leading to a "
		  "decision are \"expensive\"");
    }
    if (d.op) {
	if (!use_bit_tree) {
	    if (d.op->dsc == &op_logical_not)
		lerror(loc,"\"!\" is an \"expensive\" operation");
	    if (d.op->dsc == &op_ne)
		lerror(loc,"\"!=\" is an \"expensive\" operation");
	    if (d.op->dsc == &op_gt)
		lerror(loc,"\">\" is an \"expensive\" operation");
	    if (d.op->dsc == &op_ge)
		lerror(loc,"\">=\" is an \"expensive\" operation");
	}
	refuse_expensive(loc,d.op->a,in_actions);
	refuse_expensive(loc,d.op->b,in_actions);
	refuse_expensive(loc,d.op->c,in_actions);
    }
}


void iflib_cheap(LOCATION loc,DATA d)
{
    if (cheap_only)
	refuse_expensive(loc,d,self_contained(d) == sc_no_true);
    /*
     * The test for "sc_no_true" handles the "class X" that will be attached
     * at a later processing step. Technically, we should also set "in_action"
     * on sc_yes, but "refuse_expensive" tests for this already.
     */
}
